import csv
import sqlite3
from contextlib import closing


def load_all():
    load_categories()
    load_nutrients()
    load_food()


def load_categories(db_path: str, save_path: str):
    with open(f"{save_path}/food_category.csv") as fh:
        reader = csv.DictReader(fh)

        with closing(sqlite3.connect(db_path)) as connection:
            with closing(connection.cursor()) as cursor:
                cursor.execute(
                    "CREATE TABLE IF NOT EXISTS food_categories (id TEXT PRIMARY KEY, code TEXT, description TEXT);")
                cursor.executemany('INSERT INTO food_categories VALUES(?,?,?) ON CONFLICT DO NOTHING;',
                                   [(record['id'], record['code'], record['description']) for record in reader])
                connection.commit()


def load_nutrients(db_path: str, save_path: str):
    with open(f"{save_path}/nutrient.csv") as mfh:
        reader = csv.DictReader(mfh)

        with closing(sqlite3.connect(db_path)) as connection:
            with closing(connection.cursor()) as cursor:
                cursor.execute(
                    "CREATE TABLE IF NOT EXISTS nutrients (id TEXT PRIMARY KEY, name TEXT, unit TEXT);")
                cursor.executemany('INSERT INTO nutrients VALUES(?,?,?) ON CONFLICT DO NOTHING;',
                                   [(record['id'], record['name'], record['unit_name']) for record in reader])
                connection.commit()

    with open(f"{save_path}/food_nutrient.csv") as nfh:
        reader = csv.DictReader(nfh)

        with closing(sqlite3.connect(db_path)) as connection:
            with closing(connection.cursor()) as cursor:
                cursor.execute(
                    "CREATE TABLE IF NOT EXISTS food_nutrients (id TEXT PRIMARY KEY, food_id TEXT, nutrient_id TEXT, amount REAL, FOREIGN KEY(food_id) REFERENCES foods(id), FOREIGN KEY(nutrient_id) REFERENCES nutrients(id));")
                cursor.executemany('INSERT INTO food_nutrients VALUES(?,?,?,?) ON CONFLICT DO NOTHING;',
                                   [(record['id'], record['fdc_id'], record['nutrient_id'], float(record['amount'])) for
                                    record in reader])
                connection.commit()


def load_food(db_path: str, save_path: str):
    with open(f"{save_path}/food.csv") as fh:
        reader = csv.DictReader(fh)

        with closing(sqlite3.connect(db_path)) as connection:
            with closing(connection.cursor()) as cursor:
                cursor.execute(
                    "CREATE TABLE IF NOT EXISTS foods (id TEXT PRIMARY KEY, type TEXT, description TEXT, category_id TEXT, FOREIGN KEY(category_id) REFERENCES food_categories(id));")

                cursor.executemany('INSERT INTO foods VALUES(?,?,?,?) ON CONFLICT DO NOTHING;',
                                   [(record['fdc_id'], record['data_type'], record['description'],
                                     record['food_category_id'] if record['food_category_id'] else None) for record in
                                    reader])
                connection.commit()


def run_query(db_path: str, query: str):
    with closing(sqlite3.connect(db_path)) as connection:
        connection.row_factory = sqlite3.Row

        with closing(connection.cursor()) as cursor:
            cursor.execute(query)
            rows = cursor.fetchall()
            return rows


def create_views(db_path: str):
    with closing(sqlite3.connect(db_path)) as connection:
        with closing(connection.cursor()) as cursor:
            cursor.execute(
                "CREATE VIEW IF NOT EXISTS foods_v AS SELECT foods.id, foods.type, foods.description, food_categories.description as category from foods LEFT JOIN food_categories on foods.category_id = food_categories.id;")
            cursor.execute(
                "CREATE VIEW IF NOT EXISTS food_nutrients_v AS SELECT food_nutrients.id,  food_nutrients.food_id, food_nutrients.nutrient_id, foods.type as food_type, foods.description as food, nutrients.name as nutrient, lower(nutrients.unit) as nutrient_unit , food_categories.description as food_category from food_nutrients LEFT JOIN nutrients ON nutrients.id = food_nutrients.nutrient_id LEFT JOIN foods ON foods.id = food_nutrients.food_id LEFT JOIN food_categories on foods.category_id = food_categories.id;")


import os
import tempfile
import zipfile
from pathlib import Path

import requests
from tqdm import tqdm


def download_data(url: str, save_path: str = None):
    local_filename = url.split('/')[-1]
    local_pathname = local_filename.split('.')[0]

    with requests.get(url, stream=True) as r:
        total_size_in_bytes = int(r.headers.get('content-length', 0))
        block_size = 1024
        progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)

        with tempfile.NamedTemporaryFile() as zip_fh:
            for data in r.iter_content(block_size):
                progress_bar.update(len(data))
                zip_fh.write(data)
            progress_bar.close()

            z = zipfile.ZipFile(zip_fh)

            if not save_path:
                temp_dirpath = tempfile.mkdtemp()
                full_dirpath = os.path.join(temp_dirpath, local_pathname)
            else:
                full_dirpath = os.path.join(save_path, local_pathname)

            Path(full_dirpath).mkdir(parents=True, exist_ok=True)
            z.extractall(full_dirpath)

    return full_dirpath



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fooddata']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0', 'requests>=2.24.0,<3.0.0', 'tqdm>=4.48.2,<5.0.0']

entry_points = \
{'console_scripts': ['fooddata = fooddata.cli:cli']}

setup_kwargs = {
    'name': 'fooddata',
    'version': '0.1.1',
    'description': 'Download, build, and query the USDA food database',
    'long_description': '# fooddata\n\nDownload, build, and query the USDA food database.\n\nDo you know what you are putting in your body?\n\n## Installation\n\n```shell script\n$ pip install fooddata --upgrade\n```\n\n## Usage\n\n```shell script\n$ fooddata build\n```\n\n```shell script\n$ fooddata query "SELECT * from foods WHERE category_id IS NOT NULL LIMIT 100;" --json | jq .\n```\n\n```shell script\n$ fooddata query "SELECT * from foods WHERE category_id IS NOT NULL LIMIT 100;" --json > output2.json\n```\n\n```shell script\n$ fooddata query "SELECT * from food_nutrients_v WHERE food_type IS NOT NULL AND food_category IS NOT NULL LIMIT 1000;" --json | jq .\n```\n\n## Update Database\n\nTo update the database, just run the first command again:\n\n```shell script\n$ fooddata build\n```\n\n## Note\n\nThe data comes from the USDA website:\n[https://fdc.nal.usda.gov/download-datasets.html](https://fdc.nal.usda.gov/download-datasets.html)\n',
    'author': 'Marc Ford',
    'author_email': 'mrfxyz567@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mfdeux/fooddata',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json as jsonp
import shutil

import click

from .db import create_views, load_categories, load_food, load_nutrients, run_query
from .download import download_data

DEFAULT_SOURCE_URL = 'https://fdc.nal.usda.gov/fdc-datasets/FoodData_Central_csv_2020-04-29.zip'


@click.group()
def cli():
    """
    Download and build USDA food database
    """
    pass


@cli.command()
@click.option('--db-path', default="data.db")
@click.option('--source', default=DEFAULT_SOURCE_URL)
def build(db_path: str, source: str):
    """
    Download food data from USDA website and build database
    """

    destination_dir = download_data(source)
    click.echo(click.style('Successfully downloaded data', fg='green', bold=True))

    click.echo('Loading categories into DB')
    load_categories(db_path, destination_dir)
    click.echo('Loading food into DB')
    load_food(db_path, destination_dir)
    click.echo('Loading nutrients into DB')
    load_nutrients(db_path, destination_dir)
    click.echo('Creating views in DB')
    create_views(db_path)
    click.echo(click.style('Database successfully built', fg='green', bold=True))

    shutil.rmtree(destination_dir)


@cli.command()
@click.argument('query')
@click.option('--db-path', default="data.db")
@click.option('--json/--no-json', default=False)
def query(query: str, db_path: str, json: bool):
    """
    Query database
    Args:
        query:
        json:

    Returns:

    """
    rows = run_query(db_path, query)

    formatted_rows = [dict(row) for row in rows]
    if json:
        click.echo(jsonp.dumps(formatted_rows))
    else:
        click.echo(formatted_rows)


if __name__ == '__main__':
    cli()
